
# q-expansion of newform 2790.2.a.g, downloaded from the LMFDB on 14 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2790
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [0], [1], [-1], [-3], [-4], [6], [-1], [3], [0], [1], [-4], [0], [-1], [12], [-9], [0], [-10], [-10], [-15], [-7], [5], [6], [-15], [-16], [3], [8], [-9], [-16], [21], [20], [6], [0], [2], [-3], [-16], [-13], [-16], [15], [-24], [12], [-7], [-24], [26], [-18], [5], [-25], [8], [3], [5], [3], [6], [14], [-12], [9], [-24], [-6], [-13], [8], [0], [32], [-12], [-16], [0], [14], [-18], [-10], [-10], [0], [2], [-6], [21], [-4], [11], [-19], [24], [0], [11], [-15], [14], [36], [32], [-24], [-25], [-10], [-15], [-18], [-10], [-12], [-4], [12], [-3], [-28], [21], [32], [-12], [-18], [-24], [11], [32], [38], [39], [12], [9], [2], [-16], [12], [6], [15], [26], [-13], [14], [21], [14], [-13], [-6], [41], [-3], [18], [0], [8], [50], [15], [3], [-37], [15], [11], [48], [-1], [14], [-16], [9], [-40], [38], [-51], [-19], [-39], [11], [-42], [45], [11], [6], [-46], [42], [5], [9], [-49], [42], [-4], [9], [-46], [30], [-19], [-18], [-22], [-54], [20], [15], [-22], [6], [30], [-24], [-34], [-42], [6], [12], [41], [-10], [-52], [39], [12], [-13], [24], [26], [-10], [63], [-28], [-45], [17], [35], [32], [15], [-58], [-42], [30], [42], [35], [-28], [56], [51], [-43], [-39], [-22], [12], [48], [12], [-52], [-37], [3], [30], [-21], [-25], [11], [17], [6], [-18], [-40], [-6], [-6], [-16], [-37], [-42], [-4], [-27], [-30], [35], [-4], [6], [24], [6], [47], [-58], [48], [47], [27], [5], [6], [-60], [-22], [30], [-64], [20], [50], [-63], [68], [21], [5], [12], [-9], [36], [-12], [50], [-16], [50], [-24], [-21], [-40], [51], [50], [36], [-49], [-42], [-63], [-37], [12], [36], [-70], [-73], [-48], [-52], [5], [-36], [77], [71], [30], [53], [27], [24], [68], [-36], [8], [-28], [17], [-31], [32], [-64], [45], [-10], [20], [-3], [72], [-40], [-12], [-58], [-28], [60], [-43], [6], [-46], [-6], [45], [-21], [-12], [-66], [-10], [-48], [32], [60], [51], [17], [20], [-39], [-16], [63], [80], [-52], [54], [-64], [90], [-34], [0], [12], [66], [-61], [-66], [56], [-60], [54], [-46], [-21], [74], [-73], [33], [11], [9], [62], [29], [44], [-48], [-45], [-52], [-24], [-70], [-66], [-25], [-3], [-34], [63], [41], [2], [-64], [-66], [-90], [-31], [90], [-48], [-34], [17], [36], [-78], [14], [71], [-33], [26], [50], [60], [-6], [33], [24], [-36], [53], [-39], [3], [-6], [2], [53], [-54], [-10], [47], [72], [11], [0], [18], [-76], [38], [-93], [-27], [-10], [18], [65], [48], [96], [50], [-48], [-40], [-84], [44], [-46], [59], [63], [62], [-81], [78], [50], [-45], [20], [-19], [-48], [-76], [18], [59], [-21], [-31], [-6], [84], [20], [50], [-90], [17], [-45], [62], [-66], [-66], [32], [-52], [-87], [-54], [53], [-39], [-84], [18], [56], [-63], [30], [-16], [57], [-33], [69], [-70], [-24]]
